//***************************************************************
//   File name: PayrollDemo.java
//   Reference: Chapter 15
//***************************************************************
// Topics:
//     1. A class with object and class attributes
//        and object and class methods
//     2. Creating and using constructors
//     3. Polymorphism by overloaded constructors
//**********************************************************

//******************************************
//******************************************
//            Employee class
//******************************************
//******************************************
class Employee
{
  //********************************
  //      attributes section
  //********************************
  // field variables
  private String name = "no name";
  private String address;
  private String ssn = "xxx";
  private int dependants = 1;   // Default for dependants field
  private int empNum;
  
  // Class attribute (static qualifier)
  private static int empCounter = 0;
  
  //*********************************
  // methods section - constructors
  //*********************************
  // Fully parameterized constructor for Employee objects
  public Employee(String n, String a, String s, int x)
  {
     this.name = n;
     this.address = a;
     this.ssn = s;
     this.dependants = x;
     this.empNum = ++empCounter;
  }
 
  // Nonparameterized constructor assigns only an employee number 
  public Employee()
  {
     this.empNum = ++empCounter; 
  }
  
  // Partially parameterized constructor assigns name and
  // consecutive employee number 
  public Employee(String n)
  {
     this.name = n;
     this.empNum = ++empCounter;
  }

  //*********************************
  // methods section - other methods
  //********************************* 
  public void showData()
  {
     System.out.print("number: " + this.empNum);
     System.out.print("\tname: " + this.name);
     System.out.print("\t\taddress: " + this.address);
     System.out.print("\n\t\tSSN: " + this.ssn);
     System.out.print("\t\tdependants: " + this.dependants +
                      "\n\n");
     System.out.flush();
  }
  
  // Class method is used to access a class variable without
  // an object reference
  public static void showTotal()
  {
     System.out.println("Total employees: " + empCounter); 
     System.out.println();
  }
  
} 
     
//******************************************
//******************************************
//             CLASS PayrollDemo
//******************************************
//******************************************
public class PayrollDemo
{
    public static void main(String[] args)
    {
    // First two objects of Employee class are created using the
    // parameterized constructor
    Employee emp1 = new Employee("Jane", "131 Calm Street",
                                  "263", 2);
    Employee emp2 = new Employee("Jim", "42 Curve Road", "261",
                                  6);
    
    // Third employee object is created with non-parameterized
    // constructor
    Employee emp3 = new Employee();
    
    // Fourth employee is created with partially parameterized
    // constructor
    Employee emp4 = new Employee("Mary");
    
    // Display data using an accessor method
    emp1.showData();
    emp2.showData();
    emp3.showData();
    emp4.showData();
    
    // Display total number of employees using a class method 
    Employee.showTotal();
    }
}



