//***********************************************************
// Program: ObjectArray
// Reference: Chapter 17
// Topics:
//     1. Creating and using an array of objects
//     2. Creating an object at runtime
//     3. Demonstrates dynamic binding (runtime polymorphism)
//
//   Requires:
//      1. Keyin class in the current directory
//***********************************************************

//******************************************
//******************************************
//            Employee class
//******************************************
//******************************************
class Employee
{
  //********************************
  //      attributes section
  //********************************
  // Instance fields
  private String name;
  private String address;
  private String ssn;
  private int dependents;
  private int empNum;
  
  // Class attribute (note the static qualifier)
  private static int consecNum = 0;
  
  //*********************************
  // methods section - constructors
  //*********************************
  // Fully parameterized constructor for Employee objects
  public Employee(String n, String a, String s, int x)
  {
     this.name = n;
     this.address = a;
     this.ssn = s;
     this.dependents = x;
     this.empNum = ++consecNum; // class attribute is accessed
                                // from an object method
  }

  public void showData()
  {
     System.out.print("number: " + this.empNum);
     System.out.print("\tname: " + this.name);
     System.out.print("\t\taddress: " + this.address);
     System.out.print("\n\t\tSSN: " + this.ssn);
     System.out.print("\t\tdependents: " + this.dependents +
                      "\n\n");
     System.out.flush();
  }
} // end of class Employee

class HourlyEmployee extends Employee
{
  //********************************
  //      attributes section
  //********************************
  private double hourlyWage;
  private int hoursWorked;
  
  //*********************************
  //        methods section 
  //*********************************  
  // Constructor
  public HourlyEmployee(String n, String a, String s, int x,
                        double wage, int hours) 
  {
    // Call the constructor in the super class
    super(n, a, s, x);
    // Initialize specific fields
    this.hourlyWage = wage;
    this.hoursWorked = hours;
  }
   
  public void showData()
  {
     super.showData();
     System.out.print("wage: " + this.hourlyWage);
     System.out.print("\thours: " + this.hoursWorked + "\n\n");     
  }
} // End of class HourlyEmployee      

//******************************************
//******************************************
//             Driving class
//******************************************
//******************************************
public class ObjectArray
{
    //******************************
    //          data
    //******************************
    // Define an array of type Employee with 4 elements
    static Employee[] empArray = new Employee[4];
   
    static int objType;    // Object type
   
    //*****************************
    //         methods
    //*****************************
    public static void main(String[] args)  
    {
    // Create two objects of Employee class
    Employee emp1 = new Employee("Jane", "131 Calm Street",
                                 "263", 2);
    Employee emp2 = new Employee("Jim", "42 Curve Road",
                                 "261", 6);
    
    // Create an object of the class HourlyEmployee
    HourlyEmployee emp3 =
       new HourlyEmployee ("Jack", "11 Bumpy St.", "333",
                           8, 11.00, 40); 
    
    // Fill the array of objects of type Employee with objects of
    // the super class and the subclass. 
    empArray[0] = emp1;
    empArray[1] = emp2;
    empArray[2] = emp3;   // This is an object of the subclass
    
    // The next object is entered from the keyboard.
    // The user first selects the object's class
	objType = Keyin.inInt("Type 0 for super class or 1 for " +
	                      "sub class: ");

	// Insert object in array according to user's input
    if(objType == 0)
       empArray[3] = new Employee("Lori", "222 Dumb Street",
                                   "333", 7);
    else
       empArray[3] = new HourlyEmployee ("Joe", "11 End St.",
                                         "999", 8, 11.00, 40);

    // Use runtime polymorphism with the showData() method
    empArray[0].showData();    // Object is of class Employee
    empArray[1].showData();    // So is this one
    empArray[2].showData();	   // Object of subclass  
    // The following object is either of the class HourlyEmployee
    // or of the class Employee, according to the user's input
    empArray[3].showData();
   }
}

 