//**********************************************************
//**********************************************************
// Project: Java for Engineers
// Program: VGraphics
// Reference: Chapter 27
// Topics:
//       1. Translating the frame origin to the client area
//       2. Creating and using colors
//       3. Vector graphics
//**********************************************************
//**********************************************************
// Requires: ActiveFrame class in the current directory
//**********************************************************
//**********************************************************

import java.awt.*;

//*********************************************
//*********************************************
//              driving class
//*********************************************
//*********************************************

public class VGraphics extends ActiveFrame
{
   // Class data
   int[] xCoords =  {350, 500, 500, 350};
   int[] yCoords =  {50, 150, 50, 150}; 
 
   //*********************************
   //      redefinition of paint()
   //*********************************
 
   public void paint(Graphics g)
   {
     // Adjust the frame origin to the client area
     g.translate(getInsets().left, getInsets().top);

     // Select a font
     Font f = new Font("Monospaced", Font.BOLD, 16);
     g.setFont(f);
     
     // Create and set a bright shade of red
     Color brightRed = new Color(200, 0, 0);
     g.setColor(brightRed);
     // Display a message
     g.drawString("Demonstrating vector graphics in Java",
	               30, 25);     
     // Create and set a shade of blue
     Color aBlue = new Color(0, 0, 255);
     g.setColor(aBlue);
     // Draw a filled oval
     g.fillOval(50, 60, 200, 100);
     
     // Draw a black-border rectangle
     Color aBlack = new Color(0, 0, 0);
     g.setColor(aBlack);
     g.drawRect(30, 180, 200, 50);
     
     // Draw a green polygon
     Color aGreen = new Color(0, 180,0);
     g.setColor(aGreen);    
     g.drawPolygon(xCoords, yCoords, 4);
     
     // Translate the image by adding 120 to the
     // y coordinates of the polygon
     for(int x = 0; x < 4; x++)
        yCoords[x] = yCoords[x] + 120;
        
     // Fill the translated polygon   
     g.fillPolygon(xCoords, yCoords, 4);  
   
     // Translate image again
     for(int x = 0; x < 4; x++)
        yCoords[x] = yCoords[x] + 120;
        
     // Draw a magenta polyline. Note that the
     // drawPolyline() does not close the figure
     Color aMagenta = new Color(180,0 ,180);
     g.setColor(aMagenta);    
     g.drawPolyline(xCoords, yCoords, 4);
     
     // Restore the original values in the array of
     // y coordinates 
     yCoords[0] = 50;
     yCoords[1] = 150;
     yCoords[2] = 50;
     yCoords[3] = 150;
     
     // Draw a cyan arc
     Color aCyan = new Color(0, 120 ,120);
     g.setColor(aCyan);    
     g.drawArc( 50,         // x of start point
                280,        // y of start point
                150,        // width
                100,        // height
                45,         // start angle 
                180);       // arc angle

   }

   //****************************
   //          main
   //****************************
   public static void main(String[] args)
   {
     // Create an object of the class VGraphics
     VGraphics aframe = new VGraphics();
     
     // Use methods in superclasses to modify object's defaults
     aframe.setSize(550, 500);             // Set frame size
     aframe.setLocation(200, 100);         // Set frame location
     aframe.setTitle("Vector Graphics Demo");   // Title bar text
     
     // Display frame calling the show() method of the Window
     // class
     aframe.show();
     
   }
}
   